@@ -1,43 +1,41 @@
-use pir_8_emu::isa::instruction::{AluOperationShiftOrRotateDirection, AluOperationShiftOrRotateType, InstructionJumpCondition, InstructionPortDirection,
-                                  InstructionMadrDirection, InstructionStckDirection, InstructionRegisterPair, AluOperation, Instruction};
+use pir_8_emu::isa::instruction::{InstructionLoadImmediateWideRegisterPair, AluOperationShiftOrRotateDirection, AluOperationShiftOrRotateType,
+                                  InstructionJumpCondition, InstructionPortDirection, InstructionMadrDirection, InstructionStckDirection,
+                                  InstructionRegisterPair, AluOperation, Instruction};
 
 
 #[test]
-fn madr() {
-    for &d in &[InstructionMadrDirection::Write, InstructionMadrDirection::Read] {
-        for &r in &[InstructionRegisterPair::Ab, InstructionRegisterPair::Cd] {
-            assert_eq!(Instruction::Madr { d: d, r: r }.data_length(), 0);
-        }
-    }
-}
-
-#[test]
-fn jump() {
-    for &cond in &[InstructionJumpCondition::Jmpz,
-                   InstructionJumpCondition::Jmpp,
-                   InstructionJumpCondition::Jmpg,
-                   InstructionJumpCondition::Jmpc,
-                   InstructionJumpCondition::Jmzg,
-                   InstructionJumpCondition::Jmzl,
-                   InstructionJumpCondition::Jmpl,
-                   InstructionJumpCondition::Jump] {
-        assert_eq!(Instruction::Jump(cond).data_length(), 0);
-    }
-}
-
-#[test]
-fn load_immediate() {
-    single_register(1, |r| Instruction::LoadImmediate { aaa: r });
+fn load_immediate_byte() {
+    single_register(1, |r| Instruction::LoadImmediateByte { rrr: r });
 }
 
 #[test]
 fn load_indirect() {
-    single_register(0, |r| Instruction::LoadIndirect { aaa: r });
+    single_register(0, |r| Instruction::LoadIndirect { rrr: r });
+}
+
+#[test]
+fn load_immediate_wide() {
+    assert_eq!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Ab }.data_length(), 2);
+    assert_eq!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Cd }.data_length(), 2);
+    assert_eq!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Xy }.data_length(), 2);
+    assert_eq!(Instruction::LoadImmediateWide { rr: InstructionLoadImmediateWideRegisterPair::Adr }.data_length(), 2);
+}
+
+#[test]
+fn jump() {
+    assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpz).data_length(), 0);
+    assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpp).data_length(), 0);
+    assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpg).data_length(), 0);
+    assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpc).data_length(), 0);
+    assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmzg).data_length(), 0);
+    assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmzl).data_length(), 0);
+    assert_eq!(Instruction::Jump(InstructionJumpCondition::Jmpl).data_length(), 0);
+    assert_eq!(Instruction::Jump(InstructionJumpCondition::Jump).data_length(), 0);
 }
 
 #[test]
 fn save() {
-    single_register(0, |r| Instruction::Save { aaa: r });
+    single_register(0, |r| Instruction::Save { rrr: r });
 }
 
 #[test]
@@ -66,11 +64,11 @@ fn alu_valid_shift_or_rotate() {
 
 #[test]
 fn move_() {
-    for aaa in 0..=0b111 {
-        for bbb in 0..=0b111 {
+    for qqq in 0..=0b111 {
+        for rrr in 0..=0b111 {
             assert_eq!(Instruction::Move {
-                               aaa: aaa,
-                               bbb: bbb,
+                               qqq: qqq,
+                               rrr: rrr,
                            }
                            .data_length(),
                        0);
@@ -78,25 +76,34 @@ fn move_() {
     }
 }
 
+#[test]
+fn madr() {
+    for &d in &[InstructionMadrDirection::Write, InstructionMadrDirection::Read] {
+        for &r in &[InstructionRegisterPair::Ab, InstructionRegisterPair::Cd] {
+            assert_eq!(Instruction::Madr { d: d, r: r }.data_length(), 0);
+        }
+    }
+}
+
 #[test]
 fn port() {
     single_register(0, |r| {
         Instruction::Port {
             d: InstructionPortDirection::In,
-            aaa: r,
+            rrr: r,
         }
     });
     single_register(0, |r| {
         Instruction::Port {
             d: InstructionPortDirection::Out,
-            aaa: r,
+            rrr: r,
         }
     });
 }
 
 #[test]
 fn comp() {
-    single_register(0, |r| Instruction::Comp { aaa: r });
+    single_register(0, |r| Instruction::Comp { rrr: r });
 }
 
 #[test]
