@@ -1,5 +1,5 @@
 use pir_8_emu::isa::instruction::{ParseInstructionError, Instruction};
-use self::super::{too_many_tokens_aaa, unrecognised_token};
+use self::super::{too_many_tokens_rrr, unrecognised_token};
 use rand::distributions::{Alphanumeric, Distribution};
 use self::super::super::super::alt_gp_registers;
 use pir_8_emu::isa::GeneralPurposeRegister;
@@ -34,15 +34,35 @@ fn raw() {
 }
 
 #[test]
-fn madr() {
-    static TOKENS_MADR_DIRECTION: &[&str] = &["WRITE", "READ"];
-    static TOKENS_MADR_REG_PAIR: &[&str] = &["A&B", "C&D"];
+fn load_immediate_byte() {
+    for pad_left in 1..3 {
+        for pad_right in 1..3 {
+            too_many_tokens_rrr(&format!("LOAD{e:wl$}IMM{e:wr$}BYTE", e = "", wl = pad_left, wr = pad_right));
+        }
+    }
+}
 
-    for d in TOKENS_MADR_DIRECTION {
-        for r in TOKENS_MADR_REG_PAIR {
-            for pad_left in 1..3 {
+#[test]
+fn load_indirect() {
+    for pad in 1..3 {
+        too_many_tokens_rrr(&format!("LOAD{e:w$}IND", e = "", w = pad));
+    }
+}
+
+#[test]
+fn load_immediate_wide() {
+    static TOKENS_LOAD_IMMEDIATE_WIDE_REGISTER_PAIR: &[&str] = &["A&B", "C&D", "X&Y", "ADR"];
+
+    for tok in TOKENS_LOAD_IMMEDIATE_WIDE_REGISTER_PAIR {
+        for pad_left in 1..3 {
+            for pad_center in 1..3 {
                 for pad_right in 1..3 {
-                    unrecognised_token(&format!("MADR{e:wl$}{}{e:wr$}{}", d, r, e = "", wl = pad_left, wr = pad_right),
+                    unrecognised_token(&format!("LOAD{e:wl$}IMM{e:wc$}WIDE{e:wr$}{}",
+                                                tok,
+                                                e = "",
+                                                wl = pad_left,
+                                                wc = pad_center,
+                                                wr = pad_right),
                                        &[],
                                        1..5,
                                        |_, _| true,
@@ -62,27 +82,20 @@ fn jump_clrf_halt() {
     }
 }
 
-#[test]
-fn load() {
-    static TOKENS_LOAD: &[&str] = &["IMM", "IND"];
-
-    for tok in TOKENS_LOAD {
-        for pad in 1..3 {
-            too_many_tokens_aaa(&format!("LOAD{e:w$}{}", tok, e = "", w = pad));
-        }
-    }
-}
-
 #[test]
 fn save() {
-    too_many_tokens_aaa("SAVE");
+    too_many_tokens_rrr("SAVE");
 }
 
 #[test]
 fn alu_raw() {
     for i in 0..=0b1111 {
         for pad in 1..5 {
-            unrecognised_token(&format!("ALU{e:w$}{}", i, e = "", w = pad), &[], 1..5, |_, _| true, |len, _, _| ParseInstructionError::TooManyTokens(len));
+            unrecognised_token(&format!("ALU{e:w$}{}", i, e = "", w = pad),
+                               &[],
+                               1..5,
+                               |_, _| true,
+                               |len, _, _| ParseInstructionError::TooManyTokens(len));
             unrecognised_token(&format!("ALU{e:w$}{:#0x}", i, e = "", w = pad),
                                &[],
                                1..5,
@@ -147,7 +160,7 @@ fn alu_sor() {
 #[test]
 fn move_() {
     for regs in &[GeneralPurposeRegister::defaults(), alt_gp_registers()] {
-        for aaa in regs {
+        for rrr in regs {
             for bbb in regs {
                 for pad_left in 1..3 {
                     for pad_center in 1..3 {
@@ -156,12 +169,12 @@ fn move_() {
                                 for pad_rrright in 1..3 {
                                     for token_len in 1..3 {
                                         for _ in 0..10 {
-                                            let aaa = aaa.letter();
+                                            let rrr = rrr.letter();
                                             let bbb = bbb.letter();
                                             let token: String = Alphanumeric.sample_iter(thread_rng()).take(token_len).collect();
 
                                             let instr = format!("{e:wl$}MOVE{e:wc$}{}{e:wr$}{}{e:wrr$}{}{e:wrrr$}",
-                                                                aaa,
+                                                                rrr,
                                                                 bbb,
                                                                 token,
                                                                 e = "",
@@ -188,20 +201,40 @@ fn move_() {
     }
 }
 
+#[test]
+fn madr() {
+    static TOKENS_MADR_DIRECTION: &[&str] = &["WRITE", "READ"];
+    static TOKENS_MADR_REG_PAIR: &[&str] = &["A&B", "C&D"];
+
+    for d in TOKENS_MADR_DIRECTION {
+        for r in TOKENS_MADR_REG_PAIR {
+            for pad_left in 1..3 {
+                for pad_right in 1..3 {
+                    unrecognised_token(&format!("MADR{e:wl$}{}{e:wr$}{}", d, r, e = "", wl = pad_left, wr = pad_right),
+                                       &[],
+                                       1..5,
+                                       |_, _| true,
+                                       |len, _, _| ParseInstructionError::TooManyTokens(len));
+                }
+            }
+        }
+    }
+}
+
 #[test]
 fn port() {
     static TOKENS_PORT_DIRECTION: &[&str] = &["IN", "OUT"];
 
     for d in TOKENS_PORT_DIRECTION {
         for pad in 1..3 {
-            too_many_tokens_aaa(&format!("PORT{e:wl$}{}", d, e = "", wl = pad));
+            too_many_tokens_rrr(&format!("PORT{e:wl$}{}", d, e = "", wl = pad));
         }
     }
 }
 
 #[test]
 fn comp() {
-    too_many_tokens_aaa("COMP");
+    too_many_tokens_rrr("COMP");
 }
 
 #[test]
